import java.io.*; 
import java.util.*; 
import ajparser.*;  class  AspectJCollector  extends AspectJParser {
	

	protected Statistics stats = new Statistics();

	
	static protected int failures = 0;

	

	AspectJCollector(InputStream in) {
		super(in);
	}

	

	public boolean parseFile(String filename) throws FileNotFoundException, IOException {
		return parseString(loadFile(filename));
	}

	

	private StringBuffer loadFile(String fileName) throws FileNotFoundException, IOException {
		StringBuffer fileContent = new StringBuffer();
		FileReader f = new FileReader(fileName);
		char[] buf = new char[1000];

		int sz = 0;
		int l = 0;
		while ( (sz = f.read(buf)) > 0)
		{
			fileContent.append(buf);
			l += sz;
		}

		fileContent.setLength(l);
		return fileContent;
	}

	

	boolean parseString(StringBuffer buffer) {
		StringReader reader = new StringReader(buffer.toString());
		//System.out.println(buffer.toString());
		ReInit(reader);
		try {
			long startTime = System.currentTimeMillis();
			CompilationUnit();
			long parseTime = System.currentTimeMillis() - startTime;
			return true;
		} catch (ParseException e) {
			System.out.println(e.getMessage());
			return false;
		}
	}

	
	public static void mainParseString(StringBuffer str) {
		AspectJCollector parser = new AspectJCollector(System.in);
		if(!parser.parseString(str)) {
			System.out.println("failure parsing of input String!");
			failures++;
		} else {
			parser.stats.file_count++;
		}
		parser.printStatistics();
	}

	

	public static void main (String[] args) {
		AspectJCollector parser = new AspectJCollector(System.in);
		if (args.length == 0)  {
			System.out.println(parserName + ":  Reading from standard input . . .");
			try {
				parser.CompilationUnit();
			} catch (ParseException e) {
				System.out.println(e.getMessage());
				System.out.println(parserName + ":  Encountered errors during parse.");
			}
		} else {
			for(int i = 0; i < args.length; i++) {
				try {
					System.out.println("looking for files...");
					Vector files = parser.getFileNames(new File(args[i]), null);
					for(int j = 0; j < files.size(); j++) {
						System.out.println("analyzing file: " + ((File)files.get(j)).getName());
						if(!parser.parseFile(((File)files.get(j)).getPath())) {
							System.out.println("failure parsing " + ((File)files.get(j)).getPath() + "!");
							failures++;
						} else {
							parser.stats.file_count++;
						}
					}
				} catch(FileNotFoundException e) {
					System.out.println("*** File not found: " + args[i] + ":\n" + e);
				} catch(IOException e) {
					System.out.println("*** IO error: " + args[i] + ":\n" + e);
				}
			}
		}
		parser.printStatistics();
		System.out.println("===");
		if(failures == 0)
			System.out.println("all files have been parsed");
		else
			System.out.println("" + failures + " file(s) could not been parsed");
	}

	

	protected Vector getFileNames(File file, Vector vec) {
		if(vec == null)
			vec = new Vector();

		if(file.isFile()) {
			vec.add(file);
			return vec;    		
		} else {
			System.out.println("including directory: " + file.getPath());
			File[] files = file.listFiles(new AspectJFileFilter());

			if(files != null) {
				for(int i = 0; i < files.length; i++) {
					vec = getFileNames(files[i], vec);
				}
			}
		}
		return vec;
	}

	

	protected void printStatistics() {
		stats.print();
	}

	
    
    public void AspectBody  () throws ParseException {
    	saveState();
    	aspect_flag = true;
    	class_flag = false;
    	interface_flag = false;
    	super.AspectBody();
		recoverState();
    }

	

    public void UnmodifiedAspectDeclaration() throws ParseException {
        super.UnmodifiedAspectDeclaration();
        stats.aspect_count++;
    }

	
    
    public void AspectDeclaration() throws ParseException {
        super.AspectDeclaration();
        stats.top_aspect_count++;        
    }

	

    public void NestedAspectDeclaration() throws ParseException {
        super.NestedAspectDeclaration();
        stats.nested_aspect_count++;        
    }

	
	protected boolean inter_type_method_flag = false;

	
	protected boolean inter_type_constructor_flag = false;

	
	protected boolean inter_type_field_flag = false;

	
	protected boolean aspect_variable_flag = false;

	
	protected boolean advice_flag = false;

	
	protected boolean method_flag = false;

	
	protected boolean constructor_flag = false;

	
	protected boolean aspect_flag = false;

	
	protected boolean class_flag = false;

	
	protected boolean interface_flag = false;

	
	private boolean tmp_aspect_flag = false;

	
	private boolean tmp_class_flag = false;

	
	private boolean tmp_interface_flag = false;

	
    protected int nested_block_count = 0;

	
    public void Block  () throws ParseException {
        int begin = this.token.beginLine;
        super.Block();
        int end = this.token.endLine;
        
        if(inter_type_constructor_flag && nested_block_count == 0) {
	        stats.inter_type_constructor_count++;
    	    stats.inter_type_constructor_loc += end - begin + 1;  
    	}
    }

	
    
    public void MethodDeclaration() throws ParseException {
		method_flag = true;
		super.MethodDeclaration();
		method_flag = false;
    }

	
    
    public void ConstructorDeclaration() throws ParseException {
		constructor_flag = true;
		super.ConstructorDeclaration();
		constructor_flag = false;
    }

	

    public void AdviceDeclaration() throws ParseException {
		advice_flag = true;
		super.AdviceDeclaration();
		advice_flag = false;
    }

	
    public void AspectVariableDeclarator  () throws ParseException {
        int begin = this.token.beginLine;
        super.AspectVariableDeclarator();
        int end = this.token.endLine;
        
        if(!inter_type_field_flag) {
	        stats.aspect_field_count++;
    	}
    }

	
	
    public void BasicTypePatternDot() throws ParseException {
    	if(aspect_variable_flag == true)
			inter_type_field_flag = true;
		super.BasicTypePatternDot();
    }

	
    
    public void InterTypeMethodDeclaration() throws ParseException {
		inter_type_method_flag = true;
		super.InterTypeMethodDeclaration();
		inter_type_method_flag = false;
    }

	
    
    public void InterTypeConstructorDeclaration() throws ParseException {
		inter_type_constructor_flag = true;
		super.InterTypeConstructorDeclaration();
		inter_type_constructor_flag = false;
    }

	
    
    public void ClassBody() throws ParseException {
    	saveState();
    	aspect_flag = false;   	
    	class_flag = true;
    	interface_flag = false;
    	super.ClassBody();
    	recoverState();
    }

	
    
    public void InterfaceMemberDeclaration() throws ParseException {
    	saveState();
    	aspect_flag = false; 
    	class_flag = false; 
    	interface_flag = true;
    	super.InterfaceMemberDeclaration();
		recoverState();
    }

	
    
    private void saveState() {
    	tmp_aspect_flag = aspect_flag;
    	tmp_class_flag = class_flag;
    	tmp_interface_flag = interface_flag;
    }

	
    
    private void recoverState() {
    	aspect_flag = tmp_aspect_flag;
    	class_flag = tmp_class_flag;
    	interface_flag = tmp_interface_flag;
    }

	
    public void VariableDeclarator() throws ParseException {
        int begin = this.token.beginLine;
        super.VariableDeclarator();
        int end = this.token.endLine;

        if(interface_flag && !method_flag) {
	        stats.interface_field_count++;
    	}
    }

	
    public void PointcutDeclaration() throws ParseException {
        int begin = this.token.beginLine;
        super.PointcutDeclaration();
        int end = this.token.endLine;
        
        stats.pointcut_count++;
    }

	
    public void InterTypeDeclareBody() throws ParseException {
        int begin = this.token.beginLine;
        super.InterTypeDeclareBody();
        int end = this.token.endLine;
        
        stats.inter_type_declare_count++;
    }


}
